#include <stdlib.h>
#include <stdio.h>
#include <sqlextract/format.h>
#include "json.h"
#include <lib/json.h>

struct json_formatter_state_t {
	JsonNode *root;
	int rows;
};
static void *json_format_create(void *arg);
static void json_format_destroy(void *storage);
static void json_format_begin(void *storage);
static void json_format_end(void *storage);
static void json_format_row(void *storage);
static void json_format_field(void *storage, struct arbval_t *field);

struct output_formatter_desc_t json_formatter = { json_format_create,
		json_format_destroy, json_format_begin, json_format_end,
		json_format_row, json_format_field };

static void *json_format_create(void *arg) {
	struct json_formatter_state_t *state = calloc(1,
			sizeof(struct json_formatter_state_t));
	state->root = NULL;
	state->rows = 0;
	return state;
}

static void json_format_destroy(void *storage) {
	struct json_formatter_state_t *state =
			(struct json_formatter_state_t *) storage;
	if(state->root) {
		json_delete(state->root);
		state->root = NULL;
	}
	free(state);
}

static void json_format_begin(void *storage) {
	struct json_formatter_state_t *state =
			(struct json_formatter_state_t *) storage;
	if(state->root) {
		json_delete(state->root);
	}
	state->root = NULL;
	state->rows = 0;
}

static void json_format_end(void *storage) {
	struct json_formatter_state_t *state =
			(struct json_formatter_state_t *) storage;
	if (!state->root)
		return;

	char *output = NULL;
	output = json_stringify(state->root, NULL);
	printf("%s\r\n", output);
	free(output);
	json_delete(state->root);
	state->root = NULL;
}

static void json_format_row(void *storage) {
	struct json_formatter_state_t *state =
			(struct json_formatter_state_t *) storage;
	json_format_end(storage);

	state->root = json_mkarray();
	state->rows++;
}

static void json_format_field(void *storage, struct arbval_t *field) {
	struct json_formatter_state_t *state =
			(struct json_formatter_state_t *) storage;
	JsonNode * json_field = NULL;
	switch (field->type) {
	case ARBVAL_TYPE_INTEGER:
		json_field = json_mknumber((double) field->value.integer);
		break;
	case ARBVAL_TYPE_NULL:
		json_field = json_mknull();
		break;
	case ARBVAL_TYPE_REAL:
		json_field = json_mknumber(field->value.real);
		break;
	case ARBVAL_TYPE_STRING:
		json_field = json_mkstring(field->value.string);
		break;
	}
	json_append_element(state->root, json_field);
}
